/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2007 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    wpcRAS

Description
    Calculates and write wind power coefficients

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "incompressible/singlePhaseTransportModel/singlePhaseTransportModel.H"
#include "incompressible/RAS/RASModel/RASModel.H"
#include "octree.H"
#include "octreeDataCell.H"
#include "wallFvPatch.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    timeSelector::addOptions();

    #include "setRootCase.H"
    #include "createTime.H"

    instantList timeDirs = timeSelector::select0(runTime, args);

    #include "createMesh.H"
    #include "createFields.H"
    #include "readDict.H"

    forAll(timeDirs, timeI)
    {
        runTime.setTime(timeDirs[timeI], timeI);

        Info<< "Time = " << runTime.timeName() << endl;
        mesh.readUpdate();

        Info<< "Reading field U\n" << endl;

        volVectorField U
        (
            IOobject
            (
                "U",
                runTime.timeName(),
                mesh,
                IOobject::MUST_READ,
                IOobject::AUTO_WRITE
            ),
            mesh
        );

        Info<< "Reading field p\n" << endl;

        volScalarField p
        (
            IOobject
            (
                "p",
                runTime.timeName(),
                mesh,
                IOobject::MUST_READ,
                IOobject::AUTO_WRITE
            ),
            mesh
        );

        Info<< "Reading field k\n" << endl;

        volScalarField k
        (
            IOobject
            (
                "k",
                runTime.timeName(),
                mesh,
                IOobject::MUST_READ,
                IOobject::AUTO_WRITE
            ),
            mesh
        );

        Info<< "Creating field pstatic\n" << endl;

        volScalarField pstatic
        (
            IOobject
            (
                "pstatic",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            p-2.0/3.0*k
        );

        dimensionedScalar refStaticPressure(
            "refStaticPressure",
            pstatic.dimensions(),
            pstatic[cellRefPstat]
        );

        Info << "Reference pressure: " << p[cellRefPstat] << endl; 
        Info << "Reference kinetic energy: " << k[cellRefPstat] << endl; 
        Info << "Reference static pressure: " << refStaticPressure << endl; 
        Info <<  endl; 

        Info<< "Creating field wpc\n" << endl;

        volScalarField wpc
        (
            IOobject
            (
                "wpc",
                runTime.timeName(),
                mesh,
                IOobject::NO_READ,
                IOobject::NO_WRITE
            ),
            (pstatic-refStaticPressure)/referenceDynamicPressure
        );

        forAll (mesh.boundary(), patchI)
        {
            if ((typeid(mesh.boundary()[patchI]) == typeid(wallFvPatch)) 
            && (mesh.boundary()[patchI].size() > 0) )
            {
                scalarField& WPC = wpc.boundaryField()[patchI];

                scalarField area = mesh.boundary()[patchI].magSf();
                scalar sumSrea=gSum(area);
                scalar wpcAreaWeightedAverage = sumProd(WPC,area) / sumSrea;

                Info<< "Patch " << patchI
                    << " named " << mesh.boundary()[patchI].name()
                    << " wpc : min: " << min(WPC)
                    << " ave: " << wpcAreaWeightedAverage
                    << " max: " << max(WPC)
                    << endl;
            }
        }

        pstatic.write();
        wpc.write();
    }

    Info<< "End" << endl;

    return(0);
}


// ************************************************************************* //
